@@ -29,6 +29,8 @@ from gevent import sleep, socket, Timeout as GTimeout
 from gevent.hub import Hub
 from gevent.select import poll
 from ssh2.utils import find_eol
+from ssh2.exceptions import AgentConnectionError, AgentListIdentitiesError, \
+    AgentAuthenticationError, AgentGetIdentityError
 
 from ..common import _validate_pkey_path
 from ...constants import DEFAULT_RETRIES, RETRY_DELAY
@@ -300,14 +302,15 @@ class BaseSSHClient(object):
                 identity_file)
             try:
                 self._pkey_auth(identity_file, password=self.password)
-            except Exception:
-                logger.debug("Authentication with identity file %s failed, "
-                             "continuing with other identities",
-                             identity_file)
+            except Exception as ex:
+                logger.debug(
+                    "Authentication with identity file %s failed with %s, "
+                    "continuing with other identities",
+                    identity_file, ex)
                 continue
             else:
-                logger.debug("Authentication succeeded with identity file %s",
-                             identity_file)
+                logger.info("Authentication succeeded with identity file %s",
+                            identity_file)
                 return
         raise AuthenticationError("No authentication methods succeeded")
 
@@ -318,12 +321,45 @@ class BaseSSHClient(object):
         raise NotImplementedError
 
     def auth(self):
+        if self.pkey is not None:
+            logger.debug(
+                "Proceeding with private key file authentication")
+            return self._pkey_auth(self.pkey, password=self.password)
+        if self.allow_agent:
+            try:
+                self._agent_auth()
+            except (AgentAuthenticationError, AgentConnectionError, AgentGetIdentityError,
+                    AgentListIdentitiesError) as ex:
+                logger.debug("Agent auth failed with %s "
+                             "continuing with other authentication methods", repr(ex))
+            except Exception as ex:
+                logger.error("Agent auth failed with - %s", repr(ex))
+            else:
+                logger.debug("Authentication with SSH Agent succeeded")
+                return
+        if self.identity_auth:
+            try:
+                return self._identity_auth()
+            except AuthenticationError:
+                if self.password is None:
+                    raise
+        if self.password is None:
+            msg = "No remaining authentication methods"
+            logger.error(msg)
+            raise AuthenticationError(msg)
+        logger.debug("Private key auth failed, trying password")
+        self._password_auth()
+
+    def _agent_auth(self):
         raise NotImplementedError
 
     def _password_auth(self):
         raise NotImplementedError
 
-    def _pkey_auth(self, password=None):
+    def _pkey_auth(self, pkey_file, password=None):
+        raise NotImplementedError
+
+    def _open_session(self):
         raise NotImplementedError
 
     def open_session(self):
@@ -500,9 +536,7 @@ class BaseSSHClient(object):
         raise NotImplementedError
 
     def _sftp_put(self, remote_fh, local_file):
-        with open(local_file, 'rb') as local_fh:
-            for data in local_fh:
-                self._eagain(remote_fh.write, data)
+        raise NotImplementedError
 
     def sftp_put(self, sftp, local_file, remote_file):
         raise NotImplementedError
