@@ -24,8 +24,7 @@ from gevent import sleep, spawn, get_hub, Timeout as GTimeout
 from gevent.select import POLLIN, POLLOUT
 from ssh2.error_codes import LIBSSH2_ERROR_EAGAIN
 from ssh2.exceptions import SFTPHandleError, SFTPProtocolError, \
-    Timeout as SSH2Timeout, AgentConnectionError, AgentListIdentitiesError, \
-    AgentAuthenticationError, AgentGetIdentityError
+    Timeout as SSH2Timeout
 from ssh2.session import Session, LIBSSH2_SESSION_BLOCK_INBOUND, LIBSSH2_SESSION_BLOCK_OUTBOUND
 from ssh2.sftp import LIBSSH2_FXF_READ, LIBSSH2_FXF_CREAT, LIBSSH2_FXF_WRITE, \
     LIBSSH2_FXF_TRUNC, LIBSSH2_SFTP_S_IRUSR, LIBSSH2_SFTP_S_IRGRP, \
@@ -35,8 +34,8 @@ from ssh2.sftp import LIBSSH2_FXF_READ, LIBSSH2_FXF_CREAT, LIBSSH2_FXF_WRITE, \
 from .tunnel import FORWARDER
 from ..base.single import BaseSSHClient
 from ...output import HostOutput
-from ...exceptions import AuthenticationException, SessionError, SFTPError, \
-    SFTPIOError, Timeout, SCPError, ProxyError
+from ...exceptions import SessionError, SFTPError, \
+    SFTPIOError, Timeout, SCPError, ProxyError, AuthenticationError
 from ...constants import DEFAULT_RETRIES, RETRY_DELAY
 
 
@@ -87,11 +86,9 @@ class SSHClient(BaseSSHClient):
         :type allow_agent: bool
         :param identity_auth: (Optional) set to False to disable attempting to
           authenticate with default identity files from
-          `pssh.clients.base_ssh_client.BaseSSHClient.IDENTITIES`
+          `pssh.clients.base.single.BaseSSHClient.IDENTITIES`
         :type identity_auth: bool
-        :param forward_ssh_agent: (Optional) Turn on SSH agent forwarding -
-          equivalent to `ssh -A` from the `ssh` command line utility.
-          Defaults to True if not set.
+        :param forward_ssh_agent: Unused - agent forwarding not implemented.
         :type forward_ssh_agent: bool
         :param proxy_host: Connect to target host via given proxy host.
         :type proxy_host: str
@@ -161,7 +158,7 @@ class SSHClient(BaseSSHClient):
         if not FORWARDER.started.is_set():
             FORWARDER.start()
             FORWARDER.started.wait()
-        FORWARDER.in_q.put((self._proxy_client, self.host, self.port))
+        FORWARDER.enqueue(self._proxy_client, self.host, self.port)
         proxy_local_port = FORWARDER.out_q.get()
         return proxy_local_port
 
@@ -175,6 +172,8 @@ class SSHClient(BaseSSHClient):
                 pass
             self.session = None
         self.sock = None
+        if isinstance(self._proxy_client, SSHClient):
+            self._proxy_client.disconnect()
 
     def spawn_send_keepalive(self):
         """Spawns a new greenlet that sends keep alive messages every
@@ -215,58 +214,38 @@ class SSHClient(BaseSSHClient):
             self.configure_keepalive()
             self._keepalive_greenlet = self.spawn_send_keepalive()
 
-    def auth(self):
-        if self.pkey is not None:
-            logger.debug(
-                "Proceeding with private key file authentication")
-            return self._pkey_auth(password=self.password)
-        if self.allow_agent:
-            try:
-                self.session.agent_auth(self.user)
-            except (AgentAuthenticationError, AgentConnectionError, AgentGetIdentityError,
-                    AgentListIdentitiesError) as ex:
-                logger.debug("Agent auth failed with %s"
-                             "continuing with other authentication methods", ex)
-            except Exception as ex:
-                logger.error("Unknown error during agent authentication - %s", ex)
-            else:
-                logger.debug("Authentication with SSH Agent succeeded")
-                return
-        try:
-            self._identity_auth()
-        except AuthenticationException:
-            if self.password is None:
-                raise
-            logger.debug("Private key auth failed, trying password")
-            self._password_auth()
+    def _agent_auth(self):
+        self.session.agent_auth(self.user)
 
-    def _pkey_auth(self, password=None):
+    def _pkey_auth(self, pkey_file, password=None):
         self.session.userauth_publickey_fromfile(
             self.user,
-            self.pkey,
-            passphrase=password if password is not None else '')
+            pkey_file,
+            passphrase=password if password is not None else b'')
 
     def _password_auth(self):
         try:
             self.session.userauth_password(self.user, self.password)
-        except Exception:
-            raise AuthenticationException("Password authentication failed")
+        except Exception as ex:
+            raise AuthenticationError("Password authentication failed - %s", ex)
+
+    def _open_session(self):
+        chan = self._eagain(self.session.open_session)
+        return chan
 
     def open_session(self):
         """Open new channel from session"""
         try:
-            chan = self._eagain(self.session.open_session)
+            chan = self._open_session()
         except Exception as ex:
             raise SessionError(ex)
-        # Multiple forward requests result in ChannelRequestDenied
-        # errors, flag is used to avoid this.
         if self.forward_ssh_agent and not self._forward_requested:
             if not hasattr(chan, 'request_auth_agent'):
                 warn("Requested SSH Agent forwarding but libssh2 version used "
                      "does not support it - ignoring")
                 return chan
-            self._eagain(chan.request_auth_agent)
-            self._forward_requested = True
+            # self._eagain(chan.request_auth_agent)
+            # self._forward_requested = True
         return chan
 
     def _make_output_readers(self, channel, stdout_buffer, stderr_buffer):
@@ -346,15 +325,9 @@ class SSHClient(BaseSSHClient):
     def _make_sftp(self):
         """Make SFTP client from open transport"""
         try:
-            sftp = self.session.sftp_init()
+            sftp = self._eagain(self.session.sftp_init)
         except Exception as ex:
             raise SFTPError(ex)
-        while sftp == LIBSSH2_ERROR_EAGAIN:
-            self.poll()
-            try:
-                sftp = self.session.sftp_init()
-            except Exception as ex:
-                raise SFTPError(ex)
         return sftp
 
     def _mkdir(self, sftp, directory):
@@ -435,8 +408,6 @@ class SSHClient(BaseSSHClient):
                 sftp.open, remote_file, f_flags, mode) as remote_fh:
             try:
                 self._sftp_put(remote_fh, local_file)
-                # THREAD_POOL.apply(
-                #     sftp_put, args=(self.session, remote_fh, local_file))
             except SFTPProtocolError as ex:
                 msg = "Error writing to remote file %s - %s"
                 logger.error(msg, remote_file, ex)
@@ -674,15 +645,9 @@ class SSHClient(BaseSSHClient):
 
     def _sftp_openfh(self, open_func, remote_file, *args):
         try:
-            fh = open_func(remote_file, *args)
+            fh = self._eagain(open_func, remote_file, *args)
         except Exception as ex:
             raise SFTPError(ex)
-        while fh == LIBSSH2_ERROR_EAGAIN:
-            self.poll(timeout=0.1)
-            try:
-                fh = open_func(remote_file, *args)
-            except Exception as ex:
-                raise SFTPError(ex)
         return fh
 
     def _sftp_get(self, remote_fh, local_file):
@@ -699,17 +664,18 @@ class SSHClient(BaseSSHClient):
                 LIBSSH2_FXF_READ, LIBSSH2_SFTP_S_IRUSR) as remote_fh:
             try:
                 self._sftp_get(remote_fh, local_file)
-                # Running SFTP in a thread requires a new session
-                # as session handles or any handles created by a session
-                # cannot be used simultaneously in multiple threads.
-                # THREAD_POOL.apply(
-                #     sftp_get, args=(self.session, remote_fh, local_file))
             except SFTPProtocolError as ex:
                 msg = "Error reading from remote file %s - %s"
                 logger.error(msg, remote_file, ex)
                 raise SFTPIOError(msg, remote_file, ex)
 
     def get_exit_status(self, channel):
+        """Get exit status code for channel or ``None`` if not ready.
+
+        :param channel: The channel to get status from.
+        :type channel: :py:mod:`ssh2.channel.Channel`
+        :rtype: int or ``None``
+        """
         if not channel.eof():
             return
         return channel.get_exit_status()
