@@ -120,6 +120,9 @@ class SSHClient(BaseSSHClient):
         if self.sock is not None and not self.sock.closed:
             self.sock.close()
 
+    def _agent_auth(self):
+        self.session.userauth_agent(self.user)
+
     def _keepalive(self):
         pass
 
@@ -153,38 +156,14 @@ class SSHClient(BaseSSHClient):
             raise ex
 
     def auth(self):
-        if self.pkey is not None:
-            logger.debug(
-                "Proceeding with private key file authentication")
-            return self._pkey_auth(self.password)
-        if self.allow_agent:
-            try:
-                self.session.userauth_agent(self.user)
-            except Exception as ex:
-                logger.debug(
-                    "Agent auth failed with %s, "
-                    "continuing with other authentication methods",
-                    ex)
-            else:
-                logger.debug(
-                    "Authentication with SSH Agent succeeded.")
-                return
         if self.gssapi_auth or (self.gssapi_server_identity or self.gssapi_client_identity):
             try:
-                self.session.userauth_gssapi()
+                return self.session.userauth_gssapi()
             except Exception as ex:
                 logger.error(
                     "GSSAPI authentication with server id %s and client id %s failed - %s",
-                    self.gssapi_server_identity, self.gssapi_client_identity,
-                    ex)
-        if self.identity_auth:
-            try:
-                self._identity_auth()
-            except AuthenticationError:
-                if self.password is None:
-                    raise
-        logger.debug("Private key auth failed, trying password")
-        self._password_auth()
+                    self.gssapi_server_identity, self.gssapi_client_identity, ex)
+        return super(SSHClient, self).auth()
 
     def _password_auth(self):
         if not self.password:
@@ -194,8 +173,8 @@ class SSHClient(BaseSSHClient):
         except Exception as ex:
             raise AuthenticationError("Password authentication failed - %s", ex)
 
-    def _pkey_auth(self, password=None):
-        pkey = import_privkey_file(self.pkey, passphrase=password if password is not None else '')
+    def _pkey_auth(self, pkey_file, password=None):
+        pkey = import_privkey_file(pkey_file, passphrase=password if password is not None else '')
         if self.cert_file is not None:
             logger.debug("Certificate file set - trying certificate authentication")
             self._import_cert_file(pkey)
@@ -210,19 +189,17 @@ class SSHClient(BaseSSHClient):
     def _shell(self, channel):
         return self._eagain(channel.request_shell)
 
+    def _open_session(self):
+        channel = self.session.channel_new()
+        channel.set_blocking(0)
+        self._eagain(channel.open_session)
+        return channel
+
     def open_session(self):
         """Open new channel from session."""
         logger.debug("Opening new channel on %s", self.host)
         try:
-            channel = self.session.channel_new()
-            channel.set_blocking(0)
-            while channel.open_session() == SSH_AGAIN:
-                logger.debug(
-                    "Channel open session blocked, waiting on socket..")
-                self.poll()
-                # Select on open session can dead lock without
-                # yielding event loop
-                sleep(.1)
+            channel = self._open_session()
         except Exception as ex:
             raise SessionError(ex)
         return channel
@@ -307,9 +284,11 @@ class SSHClient(BaseSSHClient):
         return channel.is_eof()
 
     def get_exit_status(self, channel):
-        """Get exit status from channel if ready else return `None`.
+        """Get exit status code for channel or ``None`` if not ready.
 
-        :rtype: int or `None`
+        :param channel: The channel to get status from.
+        :type channel: :py:mod:`ssh.channel.Channel`
+        :rtype: int or ``None``
         """
         if not channel.is_eof():
             return
